module controlblock (latchin, clk, data_addr, data_wr, coef_addr, MAC_clr, MAC_enable, output_latch, state);

input latchin;
input clk;

output reg data_wr;
output reg MAC_clr;
output reg MAC_enable;
output reg output_latch;

output reg [2:0] state;

output reg [4:0] data_addr;
reg [4:0] wr_addr;

output reg [4:0] coef_addr;
reg [5:0] counter;

always @(posedge clk)
	begin
      case (state)
        0 : if (latchin) begin		// Wait until latchin
			data_addr <= wr_addr;	// Set write address
			data_wr <= 1;			// Write latest word to data RAM
			MAC_clr <= 1;			// Clear MAC
			state <= 1;				// Go to state 1 on next clk
			end

		1 : begin
			MAC_clr <= 0;			// de-assert MAC_clr ready for calcs
			data_wr <=0;			// de-assert data_wr
			coef_addr <= 0;			// Start coef address at 0
			counter <= 0;			// Start counter at 0
			data_addr <= data_addr + 1; // Increment data_addr so reading starts one address higher
			state <= 2;
			end
			
		2 : begin
			if (coef_addr >= 1) MAC_enable <= 1;			// Enable MAC multiplying / summing
			data_addr <= data_addr +1;	// Increment read addr
			coef_addr <= coef_addr +1;	// Increment coef_addr
			counter <= counter +1;
			if (counter == 32) begin // stop (takes one more clock to stop MAC) (should be "last tap addr + 1"
			output_latch <= 1;		// Assert output_latch to latch MAC total
			state <= 3;
			end
		end
		
        3 : begin					// One clk tick after processing done
			MAC_enable <= 0;		// De-assert MAC_enable
			wr_addr <= data_addr - 1;	// Make next write address one below last read address
			output_latch <= 0;		// De-assert output_latch
			state <= 0;
			end
	endcase
end

initial begin
			output_latch <= 0;		// De-assert output_latch
			coef_addr <= 0;			// Reset coef_addr
			data_addr <= 0;
			data_wr <= 0;
			MAC_enable <= 0;
			state <= 0;
end
endmodule